/* pro2qlen.p
 *
 * mbpro dbname -p util/pro2qlen.p -param "emaillist" > /tmp/pro2qlen.err 2>&1
 *
 * this utility use dbname rather than friendlyName because the Pro2 schema is not
 * necessarily bound to the database holding the tables that Pro2 is replicating
 *
 * this utility makes no attempt to support Windows email at this time
 *
 *     Pro2 Replication Queue Report              02/17/2021 11:03:41
 *     
 *     Per Thread Queue Summary:
 *     
 *      QNum Oldest Table                     Queue Depth         Age
 *     ----- ------------------------------ -------------    --------
 *         1 n/a                                        0    00:00:00
 *         2 so-trans-d                           468,766    01:26:35
 *         3 n/a                                        0    00:00:00
 *         4 n/a                                        0    00:00:00
 *         5 n/a                                        0    00:00:00
 *     
 *     Per Thread Queue Details:
 *     
 *      QNum Table                            Queue Depth         Age
 *     ----- ------------------------------ -------------    --------
 *         2 so-trans-d                            82,937    01:26:35
 *         2 so-trans-x                            13,360    00:58:23
 *         2 so-trans                             372,469    00:53:27
 *     
 */


define temp-table tt_replQueue no-undo
  field qNum    as integer   format "->>>9"         label "QNum"
  field tblName as character format "x(30)"         label "Table"
  field qDepth  as int64     format ">,>>>,>>>,>>9" label "Queue Depth"
  field oldest  as datetime
  index qNum-idx is primary unique qNum tblName
.

define temp-table tt_replOrphan no-undo
  field qNum    as integer   format "->>>9"         label "QNum"
  field tblName as character format "x(30)"         label "Table"
  field oldest  as datetime
  index qNum-idx is primary unique qNum tblName
.

define new global shared variable pt_rptdir as character no-undo.

define variable rptFileName  as character no-undo.
define variable emailList    as character no-undo.

define variable inQ     as int64     no-undo format ">,>>>,>>>,>>9" label "Queue Depth".
define variable oldQ    as datetime  no-undo                        label "Age".
define variable oldTbl  as character no-undo format "x(30)"         label "Oldest Table".

define variable b_rq    as handle    no-undo.
define variable q_rq    as handle    no-undo.

define variable b_tbl   as handle    no-undo.
define variable q_tbl   as handle    no-undo.

define variable i       as integer no-undo.
define variable n       as integer no-undo.
define variable orphans as integer no-undo.

form
  tt_replQueue.qNum oldTbl inQ
 with
  frame qSummary
  width 132
  down
.


/* someone thought that time should be stored as a formatted string, let's fix that!
 */

function stringTime2seconds returns integer ( st as character ):
  return
    ( integer( entry( 1, st, ":" )) * 3600 ) +
    ( integer( entry( 2, st, ":" )) * 60 ) +
    ( integer( entry( 3, st, ":" )))
  .
end.


/* main body
 */


if search( "lib/protoplib.p" ) <> ? or search( "lib/protoplib.r" ) <> ? then
    run lib/protoplib.p  persistent.					/* load protop function library				*/

if search( "lib/vstlib.p" ) <> ? or search( "lib/vstlib.r" ) <> ? then
    run lib/vstlib.p     persistent.					/* load vst library (isReplTarget()...)			*/

if search( "lib/protop-cfg.p" ) <> ? or search( "lib/protop-cfg.r" ) <> ? then
    run lib/protop-cfg.p persistent.					/* initialize protop environment                	*/

if pt_rptdir = "" or pt_rptdir = ? then pt_rptdir = ".".

rptFileName = substitute( "&1/&2.&3", pt_rptdir, "pro2qlen", "rpt" ).

emailList = session:parameter.


/* sanity check - does this db have the pro2 schema?
 */


find dictdb._File no-lock where _File-Name = "replQueue" no-error.
if not available _file then
  do:
    message
      now pdbname(1) "does not contain a Pro2 schema."
     view-as
      alert-box information
    .
    quit.
  end.


/* build the list of tables being replicated
 */

create buffer b_tbl for table "dictdb.replTableXref".
create query q_tbl.

q_tbl:set-buffers( b_tbl ).
q_tbl:query-prepare( "preselect each replTableXref no-lock" ).

q_tbl:query-open.
do while q_tbl:get-next( no-lock ):

  create tt_replQueue.
  assign
    tt_replQueue.qNum    = b_tbl:buffer-field( "qThread" ):buffer-value()
    tt_replQueue.tblName = b_tbl:buffer-field( "srcTable" ):buffer-value()
    tt_replQueue.oldest  = datetime( today + 1, 0 )
    n       = max( n, tt_replQueue.qNum )
  .

end.  
q_tbl:query-close.


/* gather metrics
 */

create buffer b_rq  for table "dictdb.replQueue".
create query q_rq.

q_rq:set-buffers( b_rq ).
q_rq:query-prepare( 'preselect each replQueue no-lock where applied = "no"' ).

q_rq:query-open.
do while q_rq:get-next( no-lock ):

  find tt_replQueue where qNum = b_rq:buffer-field( "qThread" ):buffer-value() and tblName = b_rq:buffer-field( "srcTable" ):buffer-value() no-error.

  if not available tt_replQueue then
    do:
      find tt_replOrphan where tt_replOrphan.qNum = b_rq:buffer-field( "qThread" ):buffer-value() and tt_replOrphan.tblName = b_rq:buffer-field( "srcTable" ):buffer-value() no-error.
      if not available tt_replOrphan then
        do:
          create tt_replOrphan.
          assign
            orphans = orphans + 1
            tt_replOrphan.qNum    = b_rq:buffer-field( "qThread" ):buffer-value()
            tt_replOrphan.tblName = b_rq:buffer-field( "srcTable" ):buffer-value()
            tt_replOrphan.oldest  = datetime( b_rq:buffer-field( "eventDate" ):buffer-value(), stringTime2seconds( b_rq:buffer-field( "eventTime" ):buffer-value() ) * 1000 )
          .
        end.
      next.
    end.

  assign
    tt_replQueue.qDepth = qDepth + 1
    tt_replQueue.oldest = min( tt_replQueue.oldest, datetime( b_rq:buffer-field( "eventDate" ):buffer-value(), stringTime2seconds( b_rq:buffer-field( "eventTime" ):buffer-value() ) * 1000 ))
  .

end.
q_rq:query-close.


/* report the results
 */

output to value( rptFileName ).

put "Pro2 Replication Queue Report" string( now ) format "x(19)" to 62 skip.

put skip(1).
put "Per Thread Queue Summary:" skip.

do i = 1 to n:

  display
    i @ tt_replQueue.qNum
   with
    frame qSummary
  .

  assign
    inQ    = 0
    oldQ   = ?
    oldTbl = "n/a"
  .

  for each tt_replQueue where tt_replQueue.qNum = i and tt_replQueue.qDepth > 0 break by tt_replQueue.qNum by tt_replQueue.oldest:

    if first-of( tt_replQueue.qNum ) then
      assign
        oldQ   = tt_replQueue.oldest
        oldTbl = tt_replQueue.tblName
      .

    inQ = inQ + tt_replQueue.qDepth.

  end.

  display
    oldTbl
    inQ
    space(4)
    string(( if oldQ = ? then 0 else interval( now, oldQ, "seconds" )), "hh:mm:ss" ) label "     Age"
   with
    frame qSummary
  .

  down with frame qSummary.

end.

put skip(1).
put "Per Thread Queue Details:" skip.

for each tt_replQueue where tt_replQueue.qDepth > 0 break by tt_replQueue.qNum by tt_replQueue.oldest:

  display
    tt_replQueue.qNum
    tt_replQueue.tblName
    tt_replQueue.qDepth
    space(4)
    string( interval( now, tt_replQueue.oldest, "seconds" ), "hh:mm:ss" ) label "     Age"
  .

  if last-of( tt_replQueue.qNum ) then down 1.

end.

if orphans > 0 then
  do:

    put skip(1).
    put "Orphaned Tables:" skip.

    for each tt_replOrphan by tt_replOrphan.oldest:
      display
        tt_replOrphan.qNum
        tt_replOrphan.tblName
        space(4)
        string( interval( now, tt_replOrphan.oldest, "seconds" ), "hh:mm:ss" ) label "     Age"
      .
    end.

  end.

output close.

if emailList <> "" and emailList <> ? and opsys = "unix" then
  os-command value( substitute( 'cat &2 | mailx -s "&3" &1', emailList, rptFileName, "Pro2 Replication Queue Report" )).

quit.
